/*****************************************************************************/
/* grcmob.cc
/*
/*
/* Author:  Juan-Carlos Cano
/*          jucano@disca.upv.es
/*
/* This software is released into the public domain.
/* You are free to use it in any way you like.
/*
/* This software is provided "as is" with no expressed
/* or implied warranty.  I accept no liability for any
/* damage or loss of business that this software may cause.
/*
/*****************************************************************************/



#include "grcmob.h"




/*****************************************************************************/
/*                             Definitions                                   */
/*****************************************************************************/

#define NODE_FORMAT  "$node_(%d) set %c_ %.12f\n"
#define NODE_FORMAT2 "$ns_ at %.12f \"$node_(%d) setdest %.12f %.12f %.12f\"\n"

#define     min(x,y)    ((x) < (y) ? (x) : (y))
#define     max(x,y)    ((x) > (y) ? (x) : (y))


/*****************************************************************************/
/*                             Function prototipes                           */
/*****************************************************************************/

void usage (char**);
double random1 (double);
void generate_static_scen(void);
void generate_static_scen_exploration(void);
void generate_static_init_scen(void);
double length(POSITION, POSITION);

void generate_RWGMM(void);
void generate_RDGMM(void);
void generate_MGMM(void);
void generate_PGMM(void);

//void generate_EAGMM(void);

POSITION generate_edge(POSITION);


/*****************************************************************************/
/*                             Globals variables                             */
/*****************************************************************************/

double MAXX = 0.0;
double MAXY = 0.0;
double MAXTIME = 0.0;	//duration of simulation
double ACTUALTIME = 0.0;
int    GROUPS = 0;
int    NODES = 0;
unsigned int    SEED = 1;
double SPEED = 0.0;
int    PATRON = 1;
int    INITIAL = 0;

NODE node_set[100];

double distance_group_sensitivity    = 50.0;			//Group deviation sensitivity
double initial_pause_sensitivity     = 50.0;			//Initial pause
double pause_sensitivity             = 20.0;	        //Pause time
double distance_group_sensitivity_init    = 1.0;			//Group init deviation sensitivity (only used in the pursue model)

double group_speed_sensitivity       = 0.15;			//Group speed sensitivity
double group_init_motion_sensitivity = 0.15;            //Time sensitivity when a node in a group start motion



char *optarg;		//Global variable for getopt

int last_RDGMM_edge = 0;		//It controls in which edge was the last movement of the RDGMM model

int x_or_y	= 1;					//It control the las movement (x or y) for the Manhathan Model
double manhathan_sensitivity   = 0.1; //Sensitivity for manhatan movement. To avoid the visualization problem of nam

PURSUE pursue_set[100];
int number_pursues = 0;

/*****************************************************************************/
/*                             Main Function                                 */
/*****************************************************************************/


int main(int argc, char* argv[])
{
	char ch;
	int i,j;
	int index_max_time;
	double max_time,distance,time_to_reache;


    while ((ch = getopt(argc, argv, "g:n:t:x:y:s:m:p:i:")) != EOF) {

		switch (ch) {

		case 'g':
      	GROUPS = atoi(optarg);
			break;

  	 	case 'n':
         NODES = atoi(optarg);
         break;

    	case 't':
         MAXTIME = atof(optarg);
         break;

    	case 'x':
         MAXX = atof(optarg)-1;
         break;

    	case 'y':
         MAXY = atof(optarg)-1;
         break;

		case 's':
			SEED = atoi(optarg);
			break;

		case 'm':
			SPEED = atof(optarg);
			break;

		case 'p':
			PATRON = atoi(optarg);
			break;

		case 'i':
			INITIAL = atoi(optarg);
			break;


   	default:
      	usage(argv);
			printf("USAGE\n");
      	exit(1);
  		}

	}



	if(MAXX == 0.0 || MAXY == 0.0 || GROUPS == 0 || NODES == 0 || MAXTIME == 0.0) {
        usage(argv);
        exit(1);
    }



	if((GROUPS*NODES) > 100){
		printf("Maximum number of nodes = 100\n");
        exit(1);
    }


    fprintf(stdout, "#\n# groups: %d, nodes: %d,  max x = %.2f, max y: %.2f, seed: %d, max speed: %.2f\n#\n",
        GROUPS , NODES, MAXX+1, MAXY+1,SEED,SPEED);



	srand(SEED);

	switch (PATRON) {

	case 1:
		generate_RWGMM();
		break;

	case 2:
		generate_RDGMM();
		break;

	case 3:
		generate_MGMM();
		break;

	case 4:
		generate_PGMM();
		break;

	default:
   	usage(argv);
		printf("USAGE\n");
      exit(1);

	}

	return 0;
}


/*****************************************************************************/
/*     Function Name:  generate_RWGMM
/*     Summary Description: It generate the mobility scenario acording to
/*                          Random Waypoint Group Mobility Model
/*****************************************************************************/


void generate_RWGMM(void)
{
	int i,j;
	int index_max_time;
	double max_time,distance,time_to_reache;


	generate_static_scen();

	if (SPEED != 0.0){

		for(i=0;i<GROUPS;i++){

			fprintf(stdout, "#GROUP %d MOVEMENTS\n",i);

			ACTUALTIME += random1(initial_pause_sensitivity);
			//printf("-->(ACTUAL_TIME = %f)\n",ACTUALTIME);

			while (ACTUALTIME < MAXTIME) {

			    max_time=0.0;		//time needed to cover the major distance of the group
			    index_max_time=0;   //index of the node with the maximum distance

				for (j=0;j<NODES;j++){

					node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].target_position;

					if(j==0){
						node_set[(NODES*i)+j].target_position.X=random1(MAXX);
						node_set[(NODES*i)+j].target_position.Y=random1(MAXY);
						node_set[(NODES*i)+j].target_position.Z=0.0;
						node_set[(NODES*i)+j].speed = random1(SPEED);

						fprintf(stdout, NODE_FORMAT2, ACTUALTIME, (NODES*i)+j,
							node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
							node_set[(NODES*i)+j].speed);

						distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
						time_to_reache=distance/node_set[(NODES*i)+j].speed;

						if (time_to_reache > max_time){
							max_time=time_to_reache;
							index_max_time=node_set[(NODES*i)+j].index;
						}
						//printf("--> Nodo %d distance %f Time to reache %f (MAX_TIME = %f) (ACTUAL_TIME = %f)\n", j, distance, time_to_reache,max_time, ACTUALTIME);

					} else {

						node_set[(NODES*i)+j].target_position.X = node_set[(NODES*i)].target_position.X + random1(distance_group_sensitivity);
						node_set[(NODES*i)+j].target_position.Y = node_set[(NODES*i)].target_position.Y + random1(distance_group_sensitivity);

						//To avoid go out of the area
						if (node_set[(NODES*i)+j].target_position.X > MAXX)
							node_set[(NODES*i)+j].target_position.X = MAXX;
						if (node_set[(NODES*i)+j].target_position.Y > MAXY)
							node_set[(NODES*i)+j].target_position.Y = MAXY;

						node_set[(NODES*i)+j].target_position.Z = 0.0;
						node_set[(NODES*i)+j].speed = node_set[(NODES*i)].speed + random1(group_speed_sensitivity);

						fprintf(stdout, NODE_FORMAT2, (ACTUALTIME+random1(group_init_motion_sensitivity)), (NODES*i)+j,
							node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
							node_set[(NODES*i)+j].speed);

						distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
						time_to_reache=distance/node_set[(NODES*i)+j].speed;
						if (time_to_reache > max_time){
							max_time=time_to_reache;
							index_max_time=node_set[(NODES*i)+j].index;
						}
						//printf("-> Nodo %d distance %f Time to reache %f (MAX_TIME = %f) (ACTUAL_TIME = %f)\n", j, distance, time_to_reache,max_time, ACTUALTIME);
					}
				}
				//Update de time with the maximum time in the group
				ACTUALTIME+=max_time;
				//printf("--> (ACTUAL_TIME = %f)\n",ACTUALTIME);
				ACTUALTIME+=random1(pause_sensitivity);
				//printf("--> (ACTUAL_TIME = %f)\n",ACTUALTIME);

			}
			ACTUALTIME = 0.0;   //Set actual time = 0.0 for the next group
		}

	}

}


/*****************************************************************************/
/*     Function Name:  generate_RDGMM
/*     Summary Description: It generates the mobility scenario acording to
/*                          Random Direction Group Mobility Model
/*****************************************************************************/


void generate_RDGMM(void)
{
	int i,j;
	int index_max_time;
	double max_time,distance,time_to_reache;

	POSITION edge;


	generate_static_scen();

	if (SPEED != 0.0){

		for(i=0;i<GROUPS;i++){

			fprintf(stdout, "#GROUP %d MOVEMENTS\n",i);

			ACTUALTIME += random1(initial_pause_sensitivity);
			//printf("-->(ACTUAL_TIME = %f)\n",ACTUALTIME);

			while (ACTUALTIME < MAXTIME) {

			    max_time=0.0;		//time needed to cover the major distance of the group
			    index_max_time=0;   //index of the node with the maximum distance

				for (j=0;j<NODES;j++){

					node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].target_position;

					if(j==0){
						edge = generate_edge(node_set[(NODES*i)+j].actual_position);
						node_set[(NODES*i)+j].target_position.X=edge.X;
						node_set[(NODES*i)+j].target_position.Y=edge.Y;
						node_set[(NODES*i)+j].target_position.Z=0.0;
						node_set[(NODES*i)+j].speed = random1(SPEED);

						fprintf(stdout, NODE_FORMAT2, ACTUALTIME, (NODES*i)+j,
							node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
							node_set[(NODES*i)+j].speed);

						distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
						time_to_reache=distance/node_set[(NODES*i)+j].speed;

						if (time_to_reache > max_time){
							max_time=time_to_reache;
							index_max_time=node_set[(NODES*i)+j].index;
						}
						//printf("--> Nodo %d distance %f Time to reache %f (MAX_TIME = %f) (ACTUAL_TIME = %f)\n", j, distance, time_to_reache,max_time, ACTUALTIME);

					} else {

						node_set[(NODES*i)+j].target_position.X = node_set[(NODES*i)].target_position.X + random1(distance_group_sensitivity);
						node_set[(NODES*i)+j].target_position.Y = node_set[(NODES*i)].target_position.Y + random1(distance_group_sensitivity);

						//To avoid go out of the area
						if (node_set[(NODES*i)+j].target_position.X > MAXX)
							node_set[(NODES*i)+j].target_position.X = MAXX;
						if (node_set[(NODES*i)+j].target_position.Y > MAXY)
							node_set[(NODES*i)+j].target_position.Y = MAXY;

						node_set[(NODES*i)+j].target_position.Z = 0.0;
						node_set[(NODES*i)+j].speed = node_set[(NODES*i)].speed + random1(group_speed_sensitivity);

						fprintf(stdout, NODE_FORMAT2, (ACTUALTIME+random1(group_init_motion_sensitivity)), (NODES*i)+j,
							node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
							node_set[(NODES*i)+j].speed);

						distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
						time_to_reache=distance/node_set[(NODES*i)+j].speed;
						if (time_to_reache > max_time){
							max_time=time_to_reache;
							index_max_time=node_set[(NODES*i)+j].index;
						}
						//printf("-> Nodo %d distance %f Time to reache %f (MAX_TIME = %f) (ACTUAL_TIME = %f)\n", j, distance, time_to_reache,max_time, ACTUALTIME);
					}
				}
				//Update de time with the maximum time in the group
				ACTUALTIME+=max_time;
				//printf("--> (ACTUAL_TIME = %f)\n",ACTUALTIME);
				ACTUALTIME+=random1(pause_sensitivity);
				//printf("--> (ACTUAL_TIME = %f)\n",ACTUALTIME);

			}
			ACTUALTIME = 0.0;   //Set actual time = 0.0 for the next group
		}

	}

}


/*****************************************************************************/
/*     Function Name:  generate_MGMM
/*     Summary Description: It generate the mobility scenario acording to
/*                          Manhatan Group Mobility Model
/*****************************************************************************/


void generate_MGMM(void)
{
	int i,j;
	int index_max_time;
	double max_time,distance,time_to_reache;


	generate_static_scen();

	if (SPEED != 0.0){

		for(i=0;i<GROUPS;i++){

			fprintf(stdout, "#GROUP %d MOVEMENTS\n",i);

			ACTUALTIME += random1(initial_pause_sensitivity);
			//printf("-->(ACTUAL_TIME = %f)\n",ACTUALTIME);

			while (ACTUALTIME < MAXTIME) {

			    max_time=0.0;		//time needed to cover the major distance of the group
			    index_max_time=0;   //index of the node with the maximum distance

				for (j=0;j<NODES;j++){

					node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].target_position;

					if(j==0){

						if(x_or_y==0){
							node_set[(NODES*i)+j].target_position.X=random1(MAXX);
							node_set[(NODES*i)+j].target_position.Y=node_set[(NODES*i)+j].actual_position.Y+random1(manhathan_sensitivity);
						} else {
							node_set[(NODES*i)+j].target_position.X=node_set[(NODES*i)+j].actual_position.X+random1(manhathan_sensitivity);
							node_set[(NODES*i)+j].target_position.Y=random1(MAXY);

						}
						node_set[(NODES*i)+j].target_position.Z=0.0;
						node_set[(NODES*i)+j].speed = random1(SPEED);

						fprintf(stdout, NODE_FORMAT2, ACTUALTIME, (NODES*i)+j,
							node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
							node_set[(NODES*i)+j].speed);

						distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
						time_to_reache=distance/node_set[(NODES*i)+j].speed;

						if (time_to_reache > max_time){
							max_time=time_to_reache;
							index_max_time=node_set[(NODES*i)+j].index;
						}
						//printf("--> Nodo %d distance %f Time to reache %f (MAX_TIME = %f) (ACTUAL_TIME = %f)\n", j, distance, time_to_reache,max_time, ACTUALTIME);

					} else {
						if(x_or_y==0){
							node_set[(NODES*i)+j].target_position.X = node_set[(NODES*i)].target_position.X + random1(distance_group_sensitivity);
							node_set[(NODES*i)+j].target_position.Y = node_set[(NODES*i)+j].actual_position.Y+random1(manhathan_sensitivity);
						} else {
							node_set[(NODES*i)+j].target_position.X = node_set[(NODES*i)+j].actual_position.X+random1(manhathan_sensitivity);
							node_set[(NODES*i)+j].target_position.Y = node_set[(NODES*i)].target_position.Y + random1(distance_group_sensitivity);
						}

						//To avoid go out of the area
						if (node_set[(NODES*i)+j].target_position.X > MAXX)
							node_set[(NODES*i)+j].target_position.X = MAXX;
						if (node_set[(NODES*i)+j].target_position.Y > MAXY)
							node_set[(NODES*i)+j].target_position.Y = MAXY;

						node_set[(NODES*i)+j].target_position.Z = 0.0;
						node_set[(NODES*i)+j].speed = node_set[(NODES*i)].speed + random1(group_speed_sensitivity);

						fprintf(stdout, NODE_FORMAT2, (ACTUALTIME+random1(group_init_motion_sensitivity)), (NODES*i)+j,
							node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
							node_set[(NODES*i)+j].speed);

						distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
						time_to_reache=distance/node_set[(NODES*i)+j].speed;
						if (time_to_reache > max_time){
							max_time=time_to_reache;
							index_max_time=node_set[(NODES*i)+j].index;
						}
						//printf("-> Nodo %d distance %f Time to reache %f (MAX_TIME = %f) (ACTUAL_TIME = %f)\n", j, distance, time_to_reache,max_time, ACTUALTIME);
					}
				}
				//Update de time with the maximum time in the group
				ACTUALTIME+=max_time;
				//printf("--> (ACTUAL_TIME = %f)\n",ACTUALTIME);
				ACTUALTIME+=random1(pause_sensitivity);
				if (x_or_y == 0)
					x_or_y=1;
				else
					x_or_y=0;
				//printf("--> (ACTUAL_TIME = %f)\n",ACTUALTIME);

			}
			ACTUALTIME = 0.0;   //Set actual time = 0.0 for the next group
			x_or_y=0;
		}

	}

}

/*****************************************************************************/
/*     Function Name:  generate_PGMM
/*     Summary Description: It generates the mobility scenario acording to
/*                          Pursue Group Mobility Model
/*****************************************************************************/


void generate_PGMM(void)
{
	int i,j;
	int index_max_time;
	double max_time,distance,time_to_reache;


	if (INITIAL == 0) {
		generate_static_scen();
	} else if (INITIAL == 1) {
		generate_static_init_scen();
	} else {
		printf("USAGE ERROR\n");
      exit(1);
	}



	if (SPEED != 0.0){

		for(i=0;i<GROUPS;i++){

			fprintf(stdout, "#GROUP %d MOVEMENTS\n",i);
			//printf("-->(ACTUAL_TIME = %f)\n",ACTUALTIME);

			if(i==0) {
				ACTUALTIME += random1(initial_pause_sensitivity);
				while (ACTUALTIME < MAXTIME) {

			   	max_time=0.0;		//time needed to cover the major distance of the group
			   	index_max_time=0;   //index of the node with the maximum distance

					for (j=0;j<NODES;j++){

						node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].target_position;

						if(j==0){
							node_set[(NODES*i)+j].target_position.X=random1(MAXX);
							node_set[(NODES*i)+j].target_position.Y=random1(MAXY);
							node_set[(NODES*i)+j].target_position.Z=0.0;
							node_set[(NODES*i)+j].speed = random1(SPEED);

							fprintf(stdout, NODE_FORMAT2, ACTUALTIME, (NODES*i)+j,
								node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
								node_set[(NODES*i)+j].speed);

							distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
							time_to_reache=distance/node_set[(NODES*i)+j].speed;

							if (time_to_reache > max_time){
								max_time=time_to_reache;
								index_max_time=node_set[(NODES*i)+j].index;
							}
							//printf("--> Nodo %d distance %f Time to reache %f (MAX_TIME = %f) (ACTUAL_TIME = %f)\n", j, distance, time_to_reache,max_time, ACTUALTIME);

						} else {

							node_set[(NODES*i)+j].target_position.X = node_set[(NODES*i)].target_position.X + random1(distance_group_sensitivity);
							node_set[(NODES*i)+j].target_position.Y = node_set[(NODES*i)].target_position.Y + random1(distance_group_sensitivity);

							//To avoid go out of the area
							if (node_set[(NODES*i)+j].target_position.X > MAXX)
								node_set[(NODES*i)+j].target_position.X = MAXX;
							if (node_set[(NODES*i)+j].target_position.Y > MAXY)
								node_set[(NODES*i)+j].target_position.Y = MAXY;

							node_set[(NODES*i)+j].target_position.Z = 0.0;
							node_set[(NODES*i)+j].speed = node_set[(NODES*i)].speed + random1(group_speed_sensitivity);

							fprintf(stdout, NODE_FORMAT2, (ACTUALTIME+random1(group_init_motion_sensitivity)), (NODES*i)+j,
								node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
								node_set[(NODES*i)+j].speed);

							distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
							time_to_reache=distance/node_set[(NODES*i)+j].speed;
							if (time_to_reache > max_time){
								max_time=time_to_reache;
								index_max_time=node_set[(NODES*i)+j].index;
						}
						//printf("-> Nodo %d distance %f Time to reache %f (MAX_TIME = %f) (ACTUAL_TIME = %f)\n", j, distance, time_to_reache,max_time, ACTUALTIME);
					}
				}
					//Update de time with the maximum time in the group
					ACTUALTIME+=max_time;
					//printf("--> (ACTUAL_TIME = %f)\n",ACTUALTIME);


					pursue_set[number_pursues].pursue_pos=node_set[(NODES*i)].target_position;
					pursue_set[number_pursues].time_arrival= ACTUALTIME;
					number_pursues++;

					ACTUALTIME+=random1(pause_sensitivity);
					//printf("--> (ACTUAL_TIME = %f)\n",ACTUALTIME);

				}  	//End While

			} else {	//Not the first group
				ACTUALTIME += pursue_set[number_pursues].time_arrival;
				while (ACTUALTIME < MAXTIME) {

					max_time=0.0;		//time needed to cover the major distance of the group

					for (j=0;j<NODES;j++){
						node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].target_position;


						if(j==0){
							node_set[(NODES*i)+j].target_position=pursue_set[number_pursues].pursue_pos;
							node_set[(NODES*i)+j].speed = random1(SPEED);

							fprintf(stdout, NODE_FORMAT2, ACTUALTIME, (NODES*i)+j,
								node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
								node_set[(NODES*i)+j].speed);

							distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
							time_to_reache=distance/node_set[(NODES*i)+j].speed;

							if (time_to_reache > max_time){
								max_time=time_to_reache;
								index_max_time=node_set[(NODES*i)+j].index;
							}
							//printf("--> Nodo %d distance %f Time to reache %f (MAX_TIME = %f) (ACTUAL_TIME = %f)\n", j, distance, time_to_reache,max_time, ACTUALTIME);

						} else {

							node_set[(NODES*i)+j].target_position.X = node_set[(NODES*i)].target_position.X + random1(distance_group_sensitivity);
							node_set[(NODES*i)+j].target_position.Y = node_set[(NODES*i)].target_position.Y + random1(distance_group_sensitivity);

							//To avoid go out of the area
							if (node_set[(NODES*i)+j].target_position.X > MAXX)
								node_set[(NODES*i)+j].target_position.X = MAXX;
							if (node_set[(NODES*i)+j].target_position.Y > MAXY)
								node_set[(NODES*i)+j].target_position.Y = MAXY;

							node_set[(NODES*i)+j].target_position.Z = 0.0;
							node_set[(NODES*i)+j].speed = node_set[(NODES*i)].speed + random1(group_speed_sensitivity);

							fprintf(stdout, NODE_FORMAT2, (ACTUALTIME+random1(group_init_motion_sensitivity)), (NODES*i)+j,
								node_set[(NODES*i)+j].target_position.X, node_set[(NODES*i)+j].target_position.Y,
								node_set[(NODES*i)+j].speed);

							distance=length(node_set[(NODES*i)+j].actual_position, node_set[(NODES*i)+j].target_position);
							time_to_reache=distance/node_set[(NODES*i)+j].speed;
							if (time_to_reache > max_time){
								max_time=time_to_reache;
								index_max_time=node_set[(NODES*i)+j].index;
							}
						}

					}
					//Update de time with the maximum time in the group
					pursue_set[number_pursues].time_arrival=ACTUALTIME+max_time;
					number_pursues++;

					ACTUALTIME=max((ACTUALTIME+max_time),pursue_set[number_pursues].time_arrival);




				}  	//End While


			}		//End else


			number_pursues = 0;
			ACTUALTIME = 0.0;   //Set actual time = 0.0 for the next group
		}

	}

}


/*****************************************************************************/
/*     Function Name:  usage
/*     Summary Description: It prints how to invoque the function
/*****************************************************************************/


void usage(char **argv)
{
	printf("/*****************************************************************************/\n");
	printf("/* grcmob.cc\n");
	printf("/*\n");
	printf("/*\n");
	printf("/* Author:  Juan-Carlos Cano\n");
	printf("/*          jucano@disca.upv.es\n");
	printf("/*\n");
	printf("/* This software is released into the public domain.\n");
	printf("/* You are free to use it in any way you like.\n");
	printf("/*\n");
	printf("/* This software is provided \"as is\" with no expressed\n");
	printf("/* or implied warranty.  I accept no liability for any\n");
	printf("/* damage or loss of business that this software may cause.\n");
	printf("/*\n");
	printf("/*****************************************************************************/\n");
	printf("\n usage: %s\t -g <groups> -n <nodes> -t <simulation time>\n",argv[0]);
	printf("\t\t -x <max X> -y <max Y> -s <seed> -m <max speed> -p <model> -i <initial>\n\n");
	//printf("\t\t <areas> only used for model 3\n");
	printf("\t\t model = 1 (Random WayPoint Group Mobility Model, (by default))\n");
	printf("\t\t model = 2 (Random Direction Group Mobility Model)\n");
	printf("\t\t model = 3 (Manhatan Group Mobility Model)\n");
	printf("\t\t model = 4 (Pursue Group Mobility Model)\n\n");
//	printf("\t\t model = 5 (Exploration Area Group Mobility Model)\n");

	printf("\t\t <initial> only used for model 4\n");
	printf("\t\t <initial> = 0 (Random initial position for groups, (by default))\n");
	printf("\t\t <initial> = 1 (Same initial position for groups, (by default))\n");

}


/*****************************************************************************/
/*     Function Name:  random1
/*     Summary Description: It gives a random number between 0 and the argument limit
/*****************************************************************************/


double random1(double limit)
{

	double value;
	int i;

	i=rand();
	value=(double) i/RAND_MAX;
	return(value*limit);

}


/*****************************************************************************/
/*     Function Name:  generate_static_scen
/*     Summary Description: It generates the initial postion of nodes
/*****************************************************************************/


void generate_static_scen(void)
{
	int i, j;

	for(i=0;i<GROUPS;i++) {
				fprintf(stdout, "#GROUP %d\n",i);
		for(j=0;j<NODES;j++) {
			if(j==0){

				node_set[(NODES*i)+j].index=(NODES*i)+j;
				node_set[(NODES*i)+j].initial_position.X=random1(MAXX);
				node_set[(NODES*i)+j].initial_position.Y=random1(MAXY);
				node_set[(NODES*i)+j].initial_position.Z=0.0;

				node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].initial_position;
				node_set[(NODES*i)+j].target_position=node_set[(NODES*i)+j].initial_position;



				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'X', node_set[(NODES*i)+j].initial_position.X);
				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Y', node_set[(NODES*i)+j].initial_position.Y);
				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Z', node_set[(NODES*i)+j].initial_position.Z);


			} else {

				node_set[(NODES*i)+j].index=(NODES*i)+j;
				node_set[(NODES*i)+j].initial_position.X= node_set[(NODES*i)].initial_position.X + random1(distance_group_sensitivity);
				node_set[(NODES*i)+j].initial_position.Y= node_set[(NODES*i)].initial_position.Y + random1(distance_group_sensitivity);
				node_set[(NODES*i)+j].initial_position.Z=0.0;

				node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].initial_position;
				node_set[(NODES*i)+j].target_position=node_set[(NODES*i)+j].initial_position;


				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'X', node_set[(NODES*i)+j].initial_position.X);
				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Y', node_set[(NODES*i)+j].initial_position.Y);
				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Z', node_set[(NODES*i)+j].initial_position.Z);


			}
		}
	}
}

/*****************************************************************************/
/*     Function Name:  generate_static_init_scen
/*     Summary Description: It generates the initial position of nodes
/*                          where all the group are nearby
/*****************************************************************************/

void generate_static_init_scen(void)
{
int i, j;

	for(i=0;i<GROUPS;i++) {
		fprintf(stdout, "#GROUP %d\n",i);

		if (i==0) {
			for(j=0;j<NODES;j++) {
				if(j==0){

					node_set[(NODES*i)+j].index=(NODES*i)+j;
					node_set[(NODES*i)+j].initial_position.X=random1(MAXX);
					node_set[(NODES*i)+j].initial_position.Y=random1(MAXY);
					node_set[(NODES*i)+j].initial_position.Z=0.0;

					node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].initial_position;
					node_set[(NODES*i)+j].target_position=node_set[(NODES*i)+j].initial_position;



					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'X', node_set[(NODES*i)+j].initial_position.X);
					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Y', node_set[(NODES*i)+j].initial_position.Y);
					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Z', node_set[(NODES*i)+j].initial_position.Z);


				} else {

					node_set[(NODES*i)+j].index=(NODES*i)+j;
					node_set[(NODES*i)+j].initial_position.X= node_set[(NODES*i)].initial_position.X + random1(distance_group_sensitivity);
					node_set[(NODES*i)+j].initial_position.Y= node_set[(NODES*i)].initial_position.Y + random1(distance_group_sensitivity);
					node_set[(NODES*i)+j].initial_position.Z=0.0;

					node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].initial_position;
					node_set[(NODES*i)+j].target_position=node_set[(NODES*i)+j].initial_position;


					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'X', node_set[(NODES*i)+j].initial_position.X);
					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Y', node_set[(NODES*i)+j].initial_position.Y);
					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Z', node_set[(NODES*i)+j].initial_position.Z);
				}
			}
		} else { //Not the first group
			
			for(j=0;j<NODES;j++) {
				if(j==0){
					node_set[(NODES*i)+j].index=(NODES*i)+j;
					node_set[(NODES*i)+j].initial_position.X = node_set[(NODES*(i-1))+j].initial_position.X+random1(distance_group_sensitivity_init);
					node_set[(NODES*i)+j].initial_position.Y = node_set[(NODES*(i-1))+j].initial_position.Y+random1(distance_group_sensitivity_init);
					node_set[(NODES*i)+j].initial_position.Z=0.0;

					node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].initial_position;
					node_set[(NODES*i)+j].target_position=node_set[(NODES*i)+j].initial_position;
					
					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'X', node_set[(NODES*i)+j].initial_position.X);
					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Y', node_set[(NODES*i)+j].initial_position.Y);
					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Z', node_set[(NODES*i)+j].initial_position.Z);

				} else {
				
					node_set[(NODES*i)+j].index=(NODES*i)+j;
					node_set[(NODES*i)+j].initial_position.X= node_set[(NODES*(i-1))].initial_position.X + random1(distance_group_sensitivity);
					node_set[(NODES*i)+j].initial_position.Y= node_set[(NODES*(i-1))].initial_position.Y + random1(distance_group_sensitivity);
					node_set[(NODES*i)+j].initial_position.Z=0.0;

					node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].initial_position;
					node_set[(NODES*i)+j].target_position=node_set[(NODES*i)+j].initial_position;


					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'X', node_set[(NODES*i)+j].initial_position.X);
					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Y', node_set[(NODES*i)+j].initial_position.Y);
					fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Z', node_set[(NODES*i)+j].initial_position.Z);
				
				}
			
			}


		}
	}
}


/*****************************************************************************/
/*     Function Name:  length
/*     Summary Description: It generates the length of a route
/*****************************************************************************/

double length(POSITION initial, POSITION final)
{
	double x,y;

	x=(initial.X-final.X);
	y=(initial.Y-final.Y);
	return sqrt(x*x + y*y);
}

/*****************************************************************************/
/*     Function Name:  generate_edge
/*     Summary Description: It generates a coordenate in the edge
/*****************************************************************************/

POSITION generate_edge(POSITION initial)
{
	POSITION pos;
	double aux;
	int edge;

	edge=last_RDGMM_edge;
	pos.X=0.0;
	pos.Y=0.0;

	while (edge == last_RDGMM_edge){
		aux=random1(2*MAXX+2*MAXY);

		if (aux < MAXX){
			pos.X=aux;
			pos.Y=0.1;
			edge=1;


		} else if (aux < MAXX+MAXY){
			pos.X=MAXX-random1(distance_group_sensitivity);
			pos.Y=aux-MAXX;
			edge=2;

		} else if (aux < ((2*MAXX)+MAXY)){
			pos.X=aux-(MAXX+MAXY);
			pos.Y=MAXY-random1(distance_group_sensitivity);
			edge=3;

		} else {
			pos.X=0.1;
			pos.Y=aux-((2*MAXX)+MAXY);
			edge=4;

		}
	}

	last_RDGMM_edge = edge;
	return(pos);

}


/*****************************************************************************/
/*     Function Name:  generate_EAGMM
/*     Summary Description: It generates the mobility scenario acording to
/*                          Exploration Area Group Mobility Model
/*****************************************************************************/

/*
void generate_EAGMM(void)
{
	int i,j;
	int index_max_time;
	double max_time,distance,time_to_reache;

	generate_static_scen_exploration();

	if (SPEED != 0.0){

	}
} */


/*****************************************************************************/
/*     Function Name:  generate_static_scen_exploration
/*     Summary Description: It generates the initial position of nodes
/*   								 for the exploration patron
/*****************************************************************************/

/*

void generate_static_scen_exploration(void)
{
	int i, j;

	double WIDTH;

	WIDTH=MAXX/AREAS;

	for(i=0;i<GROUPS;i++) {
				fprintf(stdout, "#GROUP %d\n",i);
		for(j=0;j<NODES;j++) {
			if(j==0){

				node_set[(NODES*i)+j].index=(NODES*i)+j;
				node_set[(NODES*i)+j].initial_position.X=random1(WIDTH)+((i%AREAS)*WIDTH);
				node_set[(NODES*i)+j].initial_position.Y=random1(MAXY);
				node_set[(NODES*i)+j].initial_position.Z=0.0;

				node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].initial_position;
				node_set[(NODES*i)+j].target_position=node_set[(NODES*i)+j].initial_position;



				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'X', node_set[(NODES*i)+j].initial_position.X);
				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Y', node_set[(NODES*i)+j].initial_position.Y);
				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Z', node_set[(NODES*i)+j].initial_position.Z);


			} else {

				node_set[(NODES*i)+j].index=(NODES*i)+j;
				node_set[(NODES*i)+j].initial_position.X= node_set[(NODES*i)].initial_position.X + random1(distance_group_sensitivity);
				node_set[(NODES*i)+j].initial_position.Y= node_set[(NODES*i)].initial_position.Y + random1(distance_group_sensitivity);
				node_set[(NODES*i)+j].initial_position.Z=0.0;

				node_set[(NODES*i)+j].actual_position=node_set[(NODES*i)+j].initial_position;
				node_set[(NODES*i)+j].target_position=node_set[(NODES*i)+j].initial_position;


				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'X', node_set[(NODES*i)+j].initial_position.X);
				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Y', node_set[(NODES*i)+j].initial_position.Y);
				fprintf(stdout, NODE_FORMAT, (NODES*i)+j, 'Z', node_set[(NODES*i)+j].initial_position.Z);


			}
		}
	}
}

*/


